﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using System.Net;
using System.IO;
using System.Text;
using System.Xml.XPath;
using System.Xml;
using System.Collections;
using System.Globalization;

/// <summary>
/// Summary description for BibsysAdapter
/// </summary>
public class BibsysAdapter
{
    private string urlPart1;
    private string urlPart2;
    private string composedUrl;
    private FileStream fs;
    public Hashtable libraryHash { get; set; }
    public CultureInfo kultur;
    ArrayList nearLibraryHolder;
    string userLat;
    string userLon;
    string nearestLibraryIconPath;
    string documentNotFetchable;
    readonly string bookCoverKey = "36ace9e071e7cbfebe529830d17ab266";
    string errorMessage;
    int imageCounter;
    int resultCounter;

    public BibsysAdapter()
    {

        //TODO: change icon paths
        nearestLibraryIconPath = @"http://upload.wikimedia.org/wikipedia/commons/c/c3/Crystal_important.png";
        documentNotFetchable = @"http://folk.ntnu.no/bjarnesl/yatta/padlock.gif";

        //Changes the double number separator from ',' to '.'
        kultur = (CultureInfo)CultureInfo.CurrentCulture.Clone();
        kultur.NumberFormat.NumberDecimalSeparator = ".";
        urlPart1 = @"http://sru.bibsys.no/services/sru?operation=searchRetrieve&version=1.1&query=";
        urlPart2 = @"&startRecord=1&maximumRecords=10&recordSchema=info:srw/schema/1/marcxml-v1.1&stylesheet=/util/MARCXML.xsl";
        if (libraryHash == null)
        {
            libraryHash = new Hashtable();
            fillLibraryHashTable();
        }
        nearLibraryHolder = new ArrayList();
    }

    public string queryForDocument(string query, string latIn, string lonIn)
    {
        imageCounter = 0;
        resultCounter = 0;
        userLat = latIn;
        userLon = lonIn;
        composedUrl = urlPart1 + query + urlPart2;
        return handleXML(composedUrl, "");
    }
    /// <summary>
    /// The method handles the xml response from a query. As parameters it takes the xml document,
    /// and the values used in the expression
    /// </summary>
    /// <param name="xmlDocument"></param>
    /// <param name="dcoumentID"></param>
    protected string handleXML(string xmlDocument, string documentID)
    {
        string returnString = @"<div class='imageDescriptor'><img src='" + documentNotFetchable + "' width='10' height='15' title='Can not be taken away from the library' alt='Can not be taken away from the library' /> = Can not be taken away from the library&nbsp;&nbsp;&nbsp;&nbsp;";
        returnString += @"<img src='" + nearestLibraryIconPath + "' width='15' height='15' border='0' title='The library nearest to you holding the document' alt='The library nearest to you holding the document' /> = The library nearest to you holding the document</div><resultSet>";
        XPathDocument docNav = new XPathDocument(composedUrl);
        XPathNavigator nav = docNav.CreateNavigator();
        XmlNamespaceManager nsMgr = new XmlNamespaceManager(nav.NameTable);
        nsMgr.AddNamespace("fn", "http://www.w3.org/2005/02/xpath-functions");
        nsMgr.AddNamespace("marcxml", "http://www.loc.gov/MARC21/slim");
        nsMgr.AddNamespace("srw", "http://www.loc.gov/zing/srw/");

        string strExpression = @"//marcxml:record";

        //Evaluate expression
        XPathNodeIterator NodeIter = nav.Select(strExpression, nsMgr);
        string tempReturnString = "";
        while (NodeIter.MoveNext())
        {
            resultCounter++;
            string recordSelection = "marcxml:datafield[@tag]";
            string coverImageString;
            XPathNodeIterator recordIterator = NodeIter.Current.Select(recordSelection, nsMgr);
            if (NodeIter.Current.SelectSingleNode("marcxml:datafield[@tag=020]/marcxml:subfield[@code='a']", nsMgr) != null)
            {
                string currentISBN = NodeIter.Current.SelectSingleNode("marcxml:datafield[@tag=020]/marcxml:subfield[@code='a']", nsMgr).Value.ToString();
                string shortenedIsbn = convertIsbn(currentISBN);
                coverImageString = generateCoverString(shortenedIsbn);
            }
            else
            {
                coverImageString = "No ISBN";
            }
            returnString += "<div class='resultHeader'></div><div class='resultContainer'><table border='0' width='800'><tr> <td width='100' height='100' align='center' >" + coverImageString + "</td><td><table border='0' width='700'>";
            while (recordIterator.MoveNext())
            {
                if (NodeIter.CurrentPosition == 67)
                {
                }
                string bookCoverString = "";
                switch (recordIterator.Current.GetAttribute("tag", "").ToString())
                {
                    case "245":
                        //documentID Title
                        tempReturnString += "<tr><td><strong>" + recordIterator.Current.Value.ToString() + "</strong></td></tr>";
                       break;
                    case "020":
                        //ISBN value not displayed  for readability
                        //returnString += "<tr><td>" + recordIterator.Current.Value.ToString() + "</td></tr>";
                        break;
                    case "852":
                        //Document library location
                        bool trondheimLibrary = false;
                        Library currentLibrary;
                        string locationSelector = "marcxml:subfield[@code]";
                        string locationString = "";
                        XPathNodeIterator locationIterator = recordIterator.Current.Select(locationSelector, nsMgr);
                        while (locationIterator.MoveNext())
                        {
                            if (locationIterator.Current.GetAttribute("code", "").ToString() == "a")
                            {
                                if (libraryHash[locationIterator.Current.Value.ToString()] != null)
                                {
                                    currentLibrary = (Library)libraryHash[locationIterator.Current.Value.ToString()];
                                    trondheimLibrary = true;
                                    locationString = "<tr><td><a href='Default.aspx?libID=" + currentLibrary.ID + "'>" + locationIterator.Current.Value.ToString() + "</a></td></tr>";
                                    addToLibraryCheckList(currentLibrary);
                                }
                                else
                                {
                                    locationString = "<tr><td>" + locationIterator.Current.Value.ToString() + "</td></tr>";
                                    trondheimLibrary = false;
                                }
                                tempReturnString += "<location>" + locationString + "</location>";
                            }
                            else if (locationIterator.Current.GetAttribute("code", "").ToString() == "z")
                            {
                                if (trondheimLibrary)
                                {
                                    string lockString = @" <img src='" + documentNotFetchable + "' border='0' align='bottom' height='15' width='10' title='Can not be taken away from library' alt='Can not be taken from the library'/>";
                                    tempReturnString = tempReturnString.Insert(tempReturnString.IndexOf("</a>") + 4, lockString);
                                }
                            }
                        }

                        break;
                    default:
                        tempReturnString += "";
                        break;
                }

            }

            if (nearLibraryHolder.Count > 0 && nearLibraryHolder[0] != null)
            {
                string nearestLibraryString = getNearestLibrary();
                if (nearestLibraryString == "KUNST")
                {
                }
                string nearestString = "<img src='" + nearestLibraryIconPath + "' width='15' height='15' border='0' title='The library nearest to you holding the document' alt='The library nearest to you holding the document'>";
                int insertAt = tempReturnString.IndexOf(@"</a>") + 4;
                tempReturnString = tempReturnString.Insert(insertAt, nearestString);

            }
            nearLibraryHolder.Clear();

            returnString += tempReturnString + "</tr></table></td><t/tr></table></div><div class='resultFooter'></div>";
            tempReturnString = "";
        }
        return returnString + @"</resultSet>" + " Images: " + imageCounter + " Total results: " + resultCounter;
        

    }


    protected void fillLibraryHashTable()
    {
        StreamReader sr;
        try
        {
            FileStream fs = new FileStream(@"C:\Inetpub\wwwroot\resources\datafiles\library.txt", FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
            sr = new StreamReader(fs);

        }
        catch (Exception e)
        {
            string errormessage = e.Message;
            throw new IOException();
        }

        string tempLine = sr.ReadLine();
        string[] separated;
        while (tempLine != null)
        {
            separated = tempLine.Split(';');
            libraryHash.Add(separated[0].ToString(), new Library(separated[0], separated[1], separated[2], separated[3], separated[4], separated[5], double.Parse(separated[6], kultur), double.Parse(separated[7], kultur)));
            tempLine = sr.ReadLine();
        }
        sr.Close();
        return;
    }

    public void addToLibraryCheckList(Library lib)
    {
        nearLibraryHolder.Add(lib);

    }

    public string getNearestLibrary()
    {
        try
        {
            if (userLat != null || userLat != null)
            {
                string ulaDot = userLat.Replace('.', ',');
                string uloDot = userLon.Replace('.', ',');
                double userDoubleLat = double.Parse(ulaDot);
                double userDoubleLon = double.Parse(uloDot);
                string[,] distanceHolder = new string[nearLibraryHolder.Count, 2];
                foreach (Library lib in nearLibraryHolder)
                {

                    double part1 = Math.Pow((userDoubleLat - lib.lat), 2);
                    double part2 = Math.Pow((userDoubleLon - lib.lon), 2);
                    distanceHolder[nearLibraryHolder.IndexOf(lib), 0] = Math.Sqrt(part1 + part2).ToString();
                    distanceHolder[nearLibraryHolder.IndexOf(lib), 1] = lib.ID.ToString();
                }
                string lowestID = distanceHolder[0, 1];
                double lowestDistance = double.Parse(distanceHolder[0, 0]);
                for (int i = 0; i <= (distanceHolder.Length / 2) - 1; i++)
                {
                    if (distanceHolder[i, 0] != null && double.Parse(distanceHolder[i, 0]) < lowestDistance)
                    {
                        lowestID = distanceHolder[i, 1];
                        lowestDistance = double.Parse(distanceHolder[i, 0]);
                    }
                }
                nearLibraryHolder.Clear();
                return lowestID;
            }
            else return "";
        }
        catch (Exception)
        {
            return "EXCEPTION";
        }
    }

    private string convertIsbn(string inputString)
    {
        string reducedIsbn = inputString.Replace("-", "").PadRight(10);
        string shortendIsbn = reducedIsbn.Substring(0, 10);
        return shortendIsbn;
    }


    private string generateCoverString(string shortIsbn)
    {

        string amazonString = @"http://images.amazon.com/images/P/" + shortIsbn + ".01.MZZZZZZZ.jpg";
        string amazonTotalString = "<a href='" + amazonString + "'><img src='" + amazonString + "' width='50' height='75'/></a>";
        string librarythingString = @"http://covers.librarything.com/devkey/" + bookCoverKey + @"/small/isbn/" + shortIsbn;
        string libraryThingTotalString = @"<a href='" + librarythingString + "'><img src='" + librarythingString + @"' width='50' height='75'/></a>";

        HttpWebRequest webRequest = WebRequest.Create(amazonString) as HttpWebRequest;
        webRequest.KeepAlive = false;
        webRequest.Method = "POST";
        webRequest.ContentType = "application/x-www-form-urlencoded";
        webRequest.ContentLength = 0;

        StreamReader amazonReader = new StreamReader(webRequest.GetResponse().GetResponseStream());
        string amazonResult = amazonReader.ReadToEnd();

        if (amazonResult != null && amazonResult.Length > 1000)
        {
            imageCounter++;
            return amazonTotalString;
        }

        HttpWebRequest webRequestLibThing = WebRequest.Create(librarythingString) as HttpWebRequest;
        webRequestLibThing.KeepAlive = false;
        webRequestLibThing.Method = "POST";
        webRequestLibThing.ContentType = "application/x-www-form-urlencoded";
        webRequestLibThing.ContentLength = 0;

        StreamReader libThingReader = new StreamReader(webRequestLibThing.GetResponse().GetResponseStream());
        string libraryThingResult = libThingReader.ReadToEnd();

        if (libraryThingResult != null && libraryThingResult.Length > 1000)
        {
            imageCounter++;
            return libraryThingTotalString;
        }
       
        return "No image";

    }
}
