﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using System.Collections.Generic;
using System.Net;
using System.IO;
using System.Text;
using System.Collections;
using System.Text.RegularExpressions;
using System.Globalization;

/// <summary>
/// Summary description for BusAdapter
/// </summary>
public class BusAdapter
{
    public string from { get; set; }
    public string to { get; set; }
    public string[] travelContainer;
    private Hashtable busStopsHash;
    public BusAdapter()
    {
        if (busStopsHash == null)
        {
            busStopsHash = new Hashtable();
            fillBusStophash2();
        }
    }

    public string[] queryBusSchedule(string fromIn, string toIn)
    {
        from = fromIn.Trim();
        from = removeLastChar(from);
        from = reduceHouseNumber(from);
        to = toIn.Trim();
        to = removeLastChar(to);
        HttpWebRequest webRequest = WebRequest.Create("http://team-trafikk.no/team_bussorakel.asp") as HttpWebRequest;
        //HttpWebRequest webRequest = WebRequest.Create("http://www.idi.ntnu.no/~tagore/cgi-bin/busstuc/busq.cgi") as HttpWebRequest;
        string question = "next five busses from " + from + " to " + to;
        webRequest.KeepAlive = false;
        webRequest.Method = "POST";
        webRequest.ContentType = "application/x-www-form-urlencoded";
        StreamWriter sw = new StreamWriter(webRequest.GetRequestStream());

        sw.Write("lang=nor&quest=" + System.Web.HttpUtility.UrlEncode(question, Encoding.GetEncoding("ISO-8859-1")));
        sw.Close();
        StreamReader responseReader = new StreamReader(webRequest.GetResponse().GetResponseStream(), Encoding.UTF7);

        string response = responseReader.ReadToEnd();
        responseReader.Close();

        if (response.Contains("The hours indicate"))
        {
            //The Following code is not needed when testing on busTUC
            int foundBeginning = response.IndexOf("clsOrakelReply");
            string messageLong = response.Substring(foundBeginning + 16);
            int foundEnd = messageLong.IndexOf("</span>");
            string messageShort = messageLong.Substring(0, foundEnd - 4);
            structureData(messageShort.Replace("<br>", "").ToLower());

            //The following code is used against busTUC
            //int foundBeginning = response.IndexOf("<body>");
            //string messageLong = response.Substring(foundBeginning + 8);
            //int foundEnd = messageLong.IndexOf("times");
            //string messageShort = messageLong.Substring(0, foundEnd + 6);
            //structureData(messageShort);
            handleLocalNames();
        }
        else if (response.Contains("I cannot find any more connections"))
        {

            return null;
        }

        return travelContainer;

    }

    private string[] structureData(string initialString)
    {
        //char[] splitter = new char[] { ' ', '.' };
        string[] splitter = new string[] { " ." };
        string[] responseArray = initialString.Split(splitter, StringSplitOptions.RemoveEmptyEntries);
        //string[] responseArray = Regex.Split(initialString, " .");
        string[] resultStringArray = new string[6];

        foreach (string line in responseArray)
        {
            determineLineType(line);

        }

        return travelContainer;
    }



    private string[] determineLineType(string line)
    {
        string[] returnString = new string[6];
        if (line.Contains(" nearest to "))
        {
            updateStationNames(line);
        }
        //Warning! The test checks for double spaces 
        else if (line.Contains("and  bus"))
        {
            returnString = handleTransfers(line);
        }
        else if (line.Contains("arrives at"))
        {
            returnString = handleDirect(line);
        }
        return returnString;
    }
    //Updates the station names. Checks if it is the from, or to, station that is mentionened, and changes this accordingly
    private void updateStationNames(string originalStationString)
    {
        int startIndex = originalStationString.IndexOf(" is ") + 4;
        string newStation = originalStationString.Substring(startIndex, originalStationString.Length - startIndex).Trim();
        if (UrlSanitize(originalStationString).ToLower().Contains(UrlSanitize(from).ToLower()) ||
            UrlSanitize(from).ToLower().Contains(UrlSanitize(originalStationString).ToLower()))
        {
            from = newStation.Trim();
        }
        else if (UrlSanitize(originalStationString).ToLower().Contains(UrlSanitize(to).ToLower()) ||
            UrlSanitize(to).ToLower().Contains(UrlSanitize(originalStationString).ToLower()))
        {
            to = newStation.Trim();
        }

    }

    //Derives the table holding departure times as well as the transfer station
    private string[] handleTransfers(string initalString)
    {

        string departureLine = "";
        string departureTime = "";
        string transferLine = "";
        string transferPoint = "";


        //Warning! The split splits on double space! 
        string[] transferContainer = initalString.Split(new string[] { "and  " }, StringSplitOptions.None);

        foreach (string transferString in transferContainer)
        {


            if (UrlSanitize(transferString.ToLower()).Contains(UrlSanitize(from.ToLower())))
            {
                departureLine = transferString.Substring(transferString.IndexOf("bus") + 4, 2).Trim().ToLower();
                departureTime = transferString.Substring(transferString.IndexOf(" at ") + 4, 9).Trim().ToLower();
            }
            else if (
                UrlSanitize(transferString.ToLower()).Contains(UrlSanitize(to.ToLower())))
            {
                transferLine = transferString.Substring(transferString.IndexOf("bus") + 4, 2).Trim().ToLower();
                transferPoint = transferString.Substring(transferString.IndexOf("from ") + 5, transferString.IndexOf(" at ") - transferString.IndexOf("from ") - 5).Trim().ToLower();
            }
        }


        travelContainer = new string[6] { from, departureLine, departureTime, transferLine, transferPoint, to };
        return new string[6];
    }

    private string[] handleDirect(string initialString)
    {
        string[] directContainer = initialString.Split(new string[] { "at " }, StringSplitOptions.None);
        string lineNumber = "";
        string arrivalTime = "";
        List<string> timeList = new List<string>();
        foreach (string timeLine in directContainer)
        {
            if (timeLine.Contains(" pm "))
            {
                timeList.Add(timeLine.Substring(timeLine.IndexOf("pm") - 6, timeLine.IndexOf("pm") + 2).Trim());
            }
            else if (timeLine.Contains(" am "))
            {
                timeList.Add(timeLine.Substring(timeLine.IndexOf("am") - 6, timeLine.IndexOf("am") + 2).Trim());
            }
            else if (timeLine.Contains("bus ") || timeLine.Contains("Bus "))
            {
                lineNumber = timeLine.Substring(timeLine.IndexOf("bus") + 4, 2).Trim();
            }
            else if (timeLine.Contains("minutes"))
            {
                //This metod is not complete, does not handle non-accurate times (e.g.  "5-13 minutes later")
                arrivalTime = timeLine.Substring(timeLine.IndexOf(",") + 1, 4).Trim();
            }
        }
        List<string> returnTotal = new List<string>();
        returnTotal.Add(lineNumber);
        foreach (string time in timeList)
        {
            returnTotal.Add(time);
        }
        //The addition of arrivalTime to the returnTotal is excluded until non-accurate times are handled
        //returnTotal.Add(arrivalTime);

        travelContainer = new string[6] { from, lineNumber, timeList[0], null, null, to };
        return new string[6];
    }
    /// <summary>
    /// The method handles local names. Add known instances to update the to and from station immediately before returning from the query
    /// </summary>
    public void handleLocalNames()
    {
        if (busStopsHash[UrlSanitize(from).ToLower().Trim()] != null)
        {
            //The method is used for the second fillhashtable2 method
            travelContainer[0] = busStopsHash[from.ToLower()].ToString();

            //The method is used for the original fillhashtable method
            //travelContainer[0] = ((busStop)busStopsHash[from.ToLower()]).address;
        }
        if (busStopsHash[UrlSanitize(to).ToLower().Trim()] != null)
        {

            //The method is used for the second fillhashtable2 method
            travelContainer[5] = busStopsHash[to.ToLower()].ToString();
            //The method is used for the original fillhashtable method
            //travelContainer[5] = ((busStop)busStopsHash[to.ToLower()]).address;

        }
    }

    //protected void fillBusStophash()
    //{
    //    StreamReader sr;
    //    try
    //    {
    //        FileStream fs = new FileStream(@"C:\Inetpub\wwwroot\resources\datafiles\busstops.txt", FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
    //        sr = new StreamReader(fs);
    //    }
    //    catch (Exception e)
    //    {
    //        string errormessage = e.Message;
    //        throw new IOException();
    //    }
    //    string[] separated = new string[5];
    //    string tempLine = sr.ReadLine();
    //    while (tempLine != null)
    //    {
    //        if (tempLine.StartsWith("streetstat"))
    //        {
    //            //removes "streetstat(" from the beginning, and ")." from the end
    //            tempLine = tempLine.Substring(11, tempLine.Length - 13).Replace('_', ' ').Replace('\'', ' ').Trim();
    //            separated = tempLine.Split(',');
    //            try
    //            {
    //                busStopsHash.Add(separated[1].ToString().ToLower(), new busStop(separated[4], separated[1], int.Parse(separated[2]), int.Parse(separated[3])));
    //            }
    //            catch (System.ArgumentException ae)
    //            {
    //                //The key was already added, it is ignored. 

    //            }
    //        }

    //        tempLine = sr.ReadLine();
    //    }
    //    //fs.Close();
    //    sr.Close();
    //    return;
    //}

    protected void fillBusStophash2()
    {
        StreamReader sr;
        try
        {
            FileStream fs = new FileStream(@"C:\Inetpub\wwwroot\resources\datafiles\busStopNames.txt", FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
            sr = new StreamReader(fs);
        }
        catch (Exception e)
        {
            string errormessage = e.Message;
            throw new IOException();
        }
        string[] separated = new string[2];
        string tempLine = sr.ReadLine();
        while (tempLine != null)
        {
            //Used on the second fillhashtable
            separated = tempLine.Split(';');
            //used on the first fillhashtable
            //separated = tempLine.Split(',');
            try
            {
                busStopsHash.Add(separated[0].ToString().ToLower(), separated[1].ToLower());
            }
            catch (System.ArgumentException ae)
            {
                //The key was already added, it is ignored. Error is supressed.

            }
            tempLine = sr.ReadLine();
        }
        sr.Close();
        return;
    }


    public static string UrlSanitize(string url)
    {
        url = Regex.Replace(url, @"\s+", " ");
        string stFormD = url.Normalize(NormalizationForm.FormD);
        StringBuilder sb = new StringBuilder();

        for (int ich = 0; ich < stFormD.Length; ich++)
        {
            UnicodeCategory uc = CharUnicodeInfo.GetUnicodeCategory(stFormD[ich]);
            if (uc != UnicodeCategory.NonSpacingMark)
            {
                sb.Append(stFormD[ich]);
            }
        }
        string encoded = sb.ToString().Trim();
        return encoded;
    }
    public string removeLastChar(string input)
    {
        char alfa = input[input.Length - 2];
        string output = input;
        if (char.IsNumber(alfa))
        {
            char beta = input[input.Length - 1];
            if (!char.IsNumber(beta))
            {
                output = input.Substring(0, input.Length - 1);
            }
        }
        return output;
    }
    public string reduceHouseNumber(string input)
    {
        char alfa = input[input.Length - 4];
        char beta = input[input.Length - 3];
        char charlie = input[input.Length - 2];
        string output = input;
        if (alfa.Equals('-') || beta.Equals('-') || charlie.Equals('-'))
        {
            int a = input.IndexOf("-");
            if ((char.IsDigit(input[a - 1]) && char.IsDigit(input[a + 1])))
            {
                output = input.Substring(0, a);
            }
        }
        return output;
    }


}
